package com.huiju.alicloud.comm;

import com.aliyun.oss.OSSClient;
import com.huiju.alicloud.config.AliConfig;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.context.properties.EnableConfigurationProperties;
import org.springframework.context.annotation.Configuration;
import org.springframework.web.multipart.MultipartFile;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.Random;

@Configuration
@EnableConfigurationProperties(AliConfig.class)
public class AliUtil {
    private static AliConfig config;

    @Autowired
    public AliUtil(AliConfig aliConfig) {
        config = aliConfig;
    }

    public static String upload(String bucketName, MultipartFile mf, String ... toDir) throws Exception {
        if (null == mf) {
            throw new Exception("未找到上传文件");
        }

        AliConfig.OSS.Bucket bucket = getBucketByName(bucketName);
        if (null == bucket) {
            throw new Exception("未找到对应上传的 Bucket");
        }

        String orignName = mf.getOriginalFilename();
        String preFix = String.valueOf(System.currentTimeMillis());
        String fileName = preFix + "-" + ifNvl(orignName, random(32) + ".png");

        if (null != toDir && toDir.length > 0) {
            String dir = toDir[0];
            if (null != dir && !"".equals(dir.trim())) {
                if (!dir.endsWith("/")) dir += "/";
                fileName = dir + fileName;
            }
        }

        if (fileName.startsWith("/")) {
            fileName = fileName.substring(1);
        }

        try {
            String f = uploadFile(bucket, new ByteArrayInputStream(mf.getBytes()), fileName);
            return f;
        } catch (IOException e) {
            e.printStackTrace();
            throw e;
        }
    }

    private static String uploadFile(AliConfig.OSS.Bucket bucket, InputStream input, String fileName) {
        OSSClient ossClient = new OSSClient(bucket.getEndpoint(), config.getAccessKeyId(), config.getAccessKeySecret());
        ossClient.putObject(bucket.getName(), fileName, input);
        ossClient.shutdown();

        // String url = ossClient.generatePresignedUrl(bucket.getName(), fileName, expiration).toString();

        String preURL = bucket.getUrl();
        if (!preURL.endsWith("/")) preURL += "/";

        return preURL + fileName;
    }

    private static AliConfig.OSS.Bucket getBucketByName(String name) {
        if (null == name || "".equals(name.trim())) return null;

        for (AliConfig.OSS.Bucket bucket: config.getOss().getBuckets()) {
            if (name.equals(bucket.getName())) return bucket;
        }

        return null;
    }

    private static String random(int length) {
        String str="abcdefghijklmnopqrstuvwxyz0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ";
        Random random = new Random();
        StringBuffer sb = new StringBuffer();
        for(int i=0;i<length;i++){
            int number=random.nextInt(62);
            sb.append(str.charAt(number));
        }
        return sb.toString();
    }

    private static String ifNvl(String str, String def) {
        if (null == str || "".equals(str.trim()) || "null".equals(str.trim()) || "undefined".equals(str.trim())) {
            return def;
        }

        return str;
    }

}
